﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace DrawGeometryPrimitives2D
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Matrix worldMatrix;
        Matrix viewMatrix;
        Matrix projectionMatrix;

        BasicEffect basicEffect;
        VertexDeclaration vertexDeclaration;
        VertexPositionColor[] pointList;
        VertexBuffer vertexBuffer;

        int points = 100;
        short[] lineStripIndices;
        short[] triangleIndices;  
        short[] quadIndices;
        short[] pentagonIndices;
        short[] hexagonIndices;
        short[] octagonIndices;

        enum PrimType  // primtype enum
        {
            Line,
            Triangle,
            Quad,
            Pentagon,
            Hexagon,
            Octagon,
        };
        PrimType typeToDraw = PrimType.Line;
        RasterizerState rasterizerState;

        GamePadState currentGamePadState;
        GamePadState lastGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            viewMatrix = Matrix.CreateLookAt(
                new Vector3(0.0f, 0.0f, 1.0f),
                Vector3.Zero,
                Vector3.Up);

            projectionMatrix = Matrix.CreateOrthographicOffCenter(
                0,
                (float)GraphicsDevice.Viewport.Width,
                (float)GraphicsDevice.Viewport.Height,
                0,
                1.0f, 1000.0f);

            basicEffect = new BasicEffect(GraphicsDevice);
            basicEffect.VertexColorEnabled = true;
            worldMatrix = Matrix.CreateTranslation(GraphicsDevice.Viewport.Width / 2f - 250,
                GraphicsDevice.Viewport.Height / 2f - 225, 0);
            basicEffect.World = worldMatrix;
            basicEffect.View = viewMatrix;
            basicEffect.Projection = projectionMatrix;

            vertexDeclaration = new VertexDeclaration(new VertexElement[]
                {
                    new VertexElement(0, VertexElementFormat.Vector3, VertexElementUsage.Position, 0),
                    new VertexElement(12, VertexElementFormat.Color, VertexElementUsage.Color, 0)
                }
            );

            InitializePointList();
            InitializeLineStrip();
            InitializeTriangle();
            InitializeQuad();
            InitializePentagon();
            InitializeHexagon();
            InitializeOctagon();

            rasterizerState = new RasterizerState();
            rasterizerState.FillMode = FillMode.WireFrame;  //기본 필모드는 wireframe로 지정
            rasterizerState.CullMode = CullMode.None;

            base.Initialize();
        }

        /// <summary>
        /// Initializes the point list.
        /// </summary>
        private void InitializePointList()  //포인트는 가로 5개 세로 5개씩 100개로 점을 찍고 각 점에 간격은 50으로 설정
        {                                   
            pointList = new VertexPositionColor[points];

            for (int x = 0; x < 10; x++)
            {
                for (int y = 0; y < 10; y++)
                {
                    pointList[(x * 10) + y] = new VertexPositionColor(
                        new Vector3(x * 50, y * 50, 0), Color.White);
                }
            }

            // Initialize the vertex buffer, allocating memory for each vertex.
            vertexBuffer = new VertexBuffer(graphics.GraphicsDevice, vertexDeclaration,
                points, BufferUsage.None);

            // Set the vertex buffer data to the array of vertices.
            vertexBuffer.SetData<VertexPositionColor>(pointList);
        }

        private void InitializeLineStrip()
        {
            // Initialize an array of indices of type short.
            lineStripIndices = new short[points];

            // Populate the array with references to indices in the vertex buffer.
            for (int i = 0; i < points; i++)
            {
                lineStripIndices[i] = (short)(i);
            }
        }


        private void InitializeTriangle()
        {
            triangleIndices = new short[3];

            triangleIndices[0] = (short)(36);
            triangleIndices[1] = (short)(52);
            triangleIndices[2] = (short)(76);
        }

        private void InitializeQuad()   
        {
            quadIndices = new short[8];

            quadIndices[0] = (short)(32);
            quadIndices[1] = (short)(72);

            quadIndices[2] = (short)(72);
            quadIndices[3] = (short)(76);

            quadIndices[4] = (short)(76);
            quadIndices[5] = (short)(36);

            quadIndices[6] = (short)(36);
            quadIndices[7] = (short)(32);
        }

        private void InitializePentagon()   
        {
            pentagonIndices = new short[10];

            pentagonIndices[0] = (short)(52);
            pentagonIndices[1] = (short)(74);

            pentagonIndices[2] = (short)(74);
            pentagonIndices[3] = (short)(66);

            pentagonIndices[4] = (short)(66);
            pentagonIndices[5] = (short)(46);

            pentagonIndices[6] = (short)(46);
            pentagonIndices[7] = (short)(34);

            pentagonIndices[8] = (short)(34);
            pentagonIndices[9] = (short)(52);
        }

        private void InitializeHexagon()   
        {
            hexagonIndices = new short[points];

            hexagonIndices[12] = (short)(14);
            hexagonIndices[13] = (short)(32);
            hexagonIndices[14] = (short)(34);

            hexagonIndices[15] = (short)(34);
            hexagonIndices[16] = (short)(36);
            hexagonIndices[17] = (short)(14);

            hexagonIndices[18] = (short)(54);
            hexagonIndices[19] = (short)(52);
            hexagonIndices[20] = (short)(74);

            hexagonIndices[21] = (short)(74);
            hexagonIndices[22] = (short)(56);
            hexagonIndices[23] = (short)(54);

            for (int x = 0; x < 2; x++)
            {
                for (int y = 0; y < 1; y++)
                {
                    hexagonIndices[(x + y) * 6] = (short)(2 * x + 34);
                    hexagonIndices[(x + y) * 6 + 1] = (short)(2 * x + 32);
                    hexagonIndices[(x + y) * 6 + 2] = (short)(2 * x + 52);

                    hexagonIndices[(x + y) * 6 + 3] = (short)(2 * x + 52);
                    hexagonIndices[(x + y) * 6 + 4] = (short)(2 * x + 54);
                    hexagonIndices[(x + y) * 6 + 5] = (short)(2 * x + 34);
                }
            }
        }

        private void InitializeOctagon()   
        {
            octagonIndices = new short[points * 6];

            octagonIndices[36] = (short)(14);
            octagonIndices[37] = (short)(32);
            octagonIndices[38] = (short)(34);

            octagonIndices[39] = (short)(52);
            octagonIndices[40] = (short)(74);
            octagonIndices[41] = (short)(54);

            octagonIndices[42] = (short)(56);
            octagonIndices[43] = (short)(76);
            octagonIndices[44] = (short)(58);

            octagonIndices[45] = (short)(16);
            octagonIndices[46] = (short)(36);
            octagonIndices[47] = (short)(38);

            for (int x = 0; x < 3; x++)
            {
                for (int y = 0; y < 1; y++)
                {
                    octagonIndices[(x + y) * 12] = (short)((20 * x) + 16);
                    octagonIndices[(x + y) * 12 + 1] = (short)((20 * x) + 14);
                    octagonIndices[(x + y) * 12 + 2] = (short)((20 * x) + 36);

                    octagonIndices[(x + y) * 12 + 3] = (short)((20 * x) + 36);
                    octagonIndices[(x + y) * 12 + 4] = (short)((20 * x) + 14);
                    octagonIndices[(x + y) * 12 + 5] = (short)((20 * x) + 34);

                    octagonIndices[(x + y) * 12 + 6] = (short)(2 * x + 34);
                    octagonIndices[(x + y) * 12 + 7] = (short)(2 * x + 32);
                    octagonIndices[(x + y) * 12 + 8] = (short)(2 * x + 54);

                    octagonIndices[(x + y) * 12 + 9] = (short)(2 * x + 54);
                    octagonIndices[(x + y) * 12 + 10] = (short)(2 * x + 32);
                    octagonIndices[(x + y) * 12 + 11] = (short)(2 * x + 52);
                }
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            CheckGamePadInput();
            CheckKeyboardInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        private void CheckGamePadInput()
        {
            lastGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);

            if (((currentGamePadState.Buttons.A == ButtonState.Pressed)) &&
                 (lastGamePadState.Buttons.A == ButtonState.Released))
            {
                typeToDraw++;
                if (typeToDraw > PrimType.Octagon)
                    typeToDraw = PrimType.Line;
            }
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                typeToDraw++;
                if (typeToDraw > PrimType.Octagon)
                    typeToDraw = PrimType.Line;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            vertexDeclaration = new VertexDeclaration(VertexPositionColor.VertexDeclaration.GetVertexElements());
            foreach (EffectPass pass in basicEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
                GraphicsDevice.RasterizerState = rasterizerState;

                switch (typeToDraw)
                {
                    case PrimType.Line:
                        DrawLineStrip();
                        break;
                    case PrimType.Triangle:
                        DrawTriangle();
                        break;
                    case PrimType.Quad:
                        DrawQuad();
                        break;
                    case PrimType.Pentagon:
                        DrawPentagon();
                        break;
                    case PrimType.Hexagon:
                        DrawHexagon();
                        break;
                    case PrimType.Octagon:
                        DrawOctagon();
                        break;
                }
            }
            base.Draw(gameTime);
        }

        private void DrawLineStrip() //line strip으로 점 그리기
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.LineStrip,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                lineStripIndices,
                0,   // first index element to read
                99   // number of primitives to draw
            );
        }

        private void DrawTriangle()  //세모 그리기에 관한 Draw정보 입력 : 점은 100개 삼각형은 1개
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.TriangleList,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                triangleIndices,
                0,   // first index element to read
                1   // number of primitives to draw
            );
        }

        private void DrawQuad()  //네모 그리기에 관한 Draw정보 입력 : 점은 100개 사각형을 표현하는 선 4개
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.LineList,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                quadIndices,
                0,   // first index element to read
                4   // number of primitives to draw
            );
        }

        private void DrawPentagon()  //오각형 그리기에 관한 Draw정보 입력 : 점은 100개 오각형을 표현하는 선 5개
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.LineList,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                pentagonIndices,
                0,   // first index element to read
                5   // number of primitives to draw
            );
        }

        private void DrawHexagon() //육각형 그리기에 관한 Draw정보 입력 : 점은 100개 육각형을 표현하는 삼각형 8개
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.TriangleList,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                hexagonIndices,
                0,   // first index element to read
                8   // number of primitives to draw
            );
        }

        private void DrawOctagon() //팔각형 그리기에 관한 Draw정보 입력 : 점은 100개 팔각형을 표현하는 삼각형 16개
        {
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(
                PrimitiveType.TriangleList,
                pointList,
                0,   // vertex buffer offset to add to each element of the index buffer
                100,   // number of vertices to draw
                octagonIndices,
                0,   // first index element to read
                16   // number of primitives to draw
            );
        }

    }
}
